#include "windows.h"
#include "dsound.h"
#include "audio.h"

static LPDIRECTSOUND							pDirectSound   = NULL;
static LPDIRECTSOUNDBUFFER						pPrimaryBuffer = NULL;
static LPDIRECTSOUNDBUFFER						pSoundBuffer   = NULL;

BOOL InitAudio(HWND hwnd, int Frequency, int UpdateFrequency)
{
    DSBUFFERDESC    SoundBufferDesc;
	WAVEFORMATEX	WaveFormat;

	// Create the direct sound interface
    if (DirectSoundCreate(NULL, &pDirectSound, NULL) != DS_OK)
	{
		TidyAudio();
		return FALSE;
	}

	// Set the cooperative level
    if (IDirectSound_SetCooperativeLevel(pDirectSound, hwnd, DSSCL_NORMAL) != DS_OK)
    {
		TidyAudio();
		return FALSE;
    }

	// Set the speaker configuration
	IDirectSound_SetSpeakerConfig(pDirectSound, DSSPEAKER_STEREO);

	// Initialise the sound buffer description
    memset(&SoundBufferDesc, 0, sizeof(DSBUFFERDESC));
	// Set the size of the structure
    SoundBufferDesc.dwSize = sizeof(DSBUFFERDESC);
	// We require a primary buffer
    SoundBufferDesc.dwFlags = DSBCAPS_PRIMARYBUFFER;

	// Create the sound buffer
    if (IDirectSound_CreateSoundBuffer(pDirectSound, &SoundBufferDesc, &pPrimaryBuffer, NULL) != DS_OK)
    {
		TidyAudio();
		return FALSE;
    }

	WaveFormat.wFormatTag		= WAVE_FORMAT_PCM;
	WaveFormat.nChannels		= 1;
	WaveFormat.nSamplesPerSec	= Frequency;
	WaveFormat.nAvgBytesPerSec	= Frequency * 2;
	WaveFormat.nBlockAlign		= 2;
	WaveFormat.wBitsPerSample	= 16;
	WaveFormat.cbSize			= 0;
	SoundBufferDesc.dwFlags		  = DSBCAPS_CTRLFREQUENCY | DSBCAPS_CTRLVOLUME;
    SoundBufferDesc.dwBufferBytes = (Frequency / UpdateFrequency) * 2 * NO_OF_AUDIO_BUFFERS;
    SoundBufferDesc.lpwfxFormat   = &WaveFormat;

    if (IDirectSound_CreateSoundBuffer(pDirectSound, &SoundBufferDesc, &pSoundBuffer, NULL) != DS_OK)
    {
		TidyAudio();
		return FALSE;
    }

	return TRUE;
}

void StopAudio(void)
{
	if (pSoundBuffer) IDirectSoundBuffer_Stop(pSoundBuffer);
}

void UpdateAudio(short *pBuffer, int BufferSize, int Frame)
{
    LPVOID  pAudio1;
    int     AudioBytes1;
    LPVOID  pAudio2;
    int     AudioBytes2;
	DWORD	WritePos;

	if (pSoundBuffer)
	{
		BufferSize *= 2;

		if (Frame == 0)
		{
			IDirectSoundBuffer_SetCurrentPosition(pSoundBuffer, 0);
			IDirectSoundBuffer_Play(pSoundBuffer, 0, 0, DSBPLAY_LOOPING);
		}

		WritePos = ((Frame + (NO_OF_AUDIO_BUFFERS >> 1)) % NO_OF_AUDIO_BUFFERS) * BufferSize;

		// Lock the sound buffer
		if (IDirectSoundBuffer_Lock(pSoundBuffer, WritePos, BufferSize, &pAudio1, &AudioBytes1, &pAudio2, &AudioBytes2, 0) == DS_OK)
		{
			// Buffer is now locked, write first block
			memcpy(pAudio1, (char *)pBuffer, AudioBytes1);

			// See if we need to write a second block
			if (AudioBytes2)
			{
				// Yes, so write second block
				memcpy(pAudio2, (char *)((int)pBuffer + AudioBytes1), AudioBytes2);
			}

			// Unlock buffer
			IDirectSoundBuffer_Unlock(pSoundBuffer, pAudio1, AudioBytes1, pAudio2, AudioBytes2);
		}
	}
}

void TidyAudio(void)
{
	if (pSoundBuffer) 
	{
		IDirectSoundBuffer_Stop(pSoundBuffer);
		IDirectSoundBuffer_Release(pSoundBuffer);
		pSoundBuffer = NULL;
	}

	if (pPrimaryBuffer) 
	{
		IDirectSoundBuffer_Stop(pPrimaryBuffer);
		IDirectSoundBuffer_Release(pPrimaryBuffer);
		pPrimaryBuffer = NULL;
	}

	if (pDirectSound)
	{
		IDirectSound_Release(pDirectSound);
		pDirectSound = NULL;
	}
}
