#include <windows.h>
#include "shared.h"
#include "dinput.h"
#include "main.h"
#include "input.h"
#include "registry.h"
#include "screenshot.h"

void Screenshot(void)
{
	int				i;
	int				AppendPos;
	char			Filename[MAX_PATH];
	HANDLE			FileHandle;

	// Add the game name (as used for SRAM save), and add it to the screenshots path
	wsprintf(Filename, "%s%s", RegistryInfo.ScreenshotPath, GetSRAMName());

	// Calculate the position where we can trim off the old .sav extension
	AppendPos = strlen(Filename) - 4;

	for (i = 0 ; i < 9999 ; i++)
	{
		// Add a numbered index and the extension .bmp
		wsprintf(&Filename[AppendPos], "(%04d).bmp", i);

		// Attempt to create a new file
	    FileHandle = CreateFile(Filename, GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_FLAG_SEQUENTIAL_SCAN, NULL);

		// If we're successful, stop looking for a new name
		if (FileHandle != INVALID_HANDLE_VALUE) break;
	}

	// Should never be the case, unless you get 10000 pics from the same game!
	if (FileHandle != INVALID_HANDLE_VALUE)
	{
		unsigned int		BytesWritten;
		BITMAPFILEHEADER	*pBMP;
		unsigned int		FileSize;	

		FileSize = sizeof(BITMAPFILEHEADER) + sizeof(BMP) + (256 * 192);

		pBMP = (BITMAPFILEHEADER *)malloc(FileSize);

		if (pBMP)
		{
			BOOL		Success;

			// Setup the bitmap file header
			pBMP->bfType	  = 'MB';
			pBMP->bfOffBits	  = sizeof(BITMAPFILEHEADER) + sizeof(BMP);
			pBMP->bfSize	  = FileSize;
			pBMP->bfReserved1 = 0;
			pBMP->bfReserved2 = 0;

			// Copy over the bitmap info header
			memcpy((char *)((int)pBMP + sizeof(BITMAPFILEHEADER)), GetBitmapInfoHeader(), sizeof(BMP));
			
			if (cart.type == TYPE_GG)
			{
				BITMAPINFOHEADER *pBMPInfo = (BITMAPINFOHEADER *)((int)pBMP + sizeof(BITMAPFILEHEADER));
				char			 *pSource  = GetBitmapData() + (24 * 256) + 48;
				char			 *pDest	   = (char *)((int)pBMP + sizeof(BITMAPFILEHEADER) + sizeof(BMP));
				int				 i;

				// Screen size is smaller for gamegear
				pBMPInfo->biWidth  = 160;
				pBMPInfo->biHeight = 144;

				// Re-build smaller bitmap
				for (i = 0 ; i < 144 ; i++)
				{
					memcpy(pDest, pSource, 160);
					pSource += 256;
					pDest	+= 160;
				}

				// Adjust file size
				FileSize -= (48 * 256) + (144 * 96);

				pBMP->bfSize = FileSize;
			}
			else
			{
				// Copy over the image data
				memcpy((char *)((int)pBMP + sizeof(BITMAPFILEHEADER) + sizeof(BMP)), GetBitmapData(), 256 * 192);
			}

			// Write out the .bmp
		    Success = WriteFile(FileHandle, pBMP, FileSize, &BytesWritten, NULL);

			// Check the file was written out properly
			if (!Success || (BytesWritten != FileSize))
			{
				char		Message[MAX_PATH];

				// Print up an error message
				wsprintf(Message, "Error writing : %s", Filename); 
				MessageBox(NULL, Message, "Error", MB_OK | MB_ICONWARNING);
			}

			// Free the allocation
			free(pBMP);
		}

		// Close the file
		CloseHandle(FileHandle);
	}
}